//********************************************************************
//*** ResourcePool/SQL/RTSQLSelectRow.cc
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: RTSQLSelectRow.cc,v 1.26 2009-03-12 18:49:39 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include "ResourcePool/SQL/ResourcePool.hh"
#include "ResourcePool/SQL/RTSQLSelectRow.hh"
#include "ResourcePool/SQL/RTSQLSelectRow_Impl.hh"

// todo, a little strange
#ifdef HAVE_ORACLE
#include "ResourcePool/oracle/oracleType.hh"
#endif

namespace fatalmind {
namespace SQL {

template<class TM>
struct SQLSelectRowCommandSelector {
	typedef RTSQLSelectRow_Impl<TM> Impl;
	typedef RTSQLSelectRow<TM> RTC;

	template<class PT>
	struct Specialized {
		typedef RTSQLSelectRow_SpecializedImpl<
			  TM
			, RTSQLSelectRow_Impl
			, PT
			, SQLCommandWrapper<typename PT::command_t::SQLSelectRow, TM, RTSQLSelectRow<TM> >
			, RTSQLSelectRow<TM>
		> Impl;
	};
};

template<class TM>
RTSQLSelectRow<TM>::RTSQLSelectRow(const ResourcePool<ResourceType<TM> >& p, const std::string& sql, SQL::SQLFetcher& fetcher)
: _realSQLExecute(
    RTSQLSelectRow_Impl<TM>::template newRealCommand<
          SQLSelectRowCommandSelector<TM>
    >(
		  Command<ResourceType<TM> >::getFactory(p).getDatabaseType()
        , *this
        , sql
		, fetcher
    )
)
{
}

template<class TM>
RTSQLSelectRow<TM>::~RTSQLSelectRow() {
}

template<class TM>
template<class T>
void RTSQLSelectRow<TM>::bind(const unsigned int pos, const T& val, bool nul) {
	_realSQLExecute->bind(pos, val, nul);
}


template<class TM>
void RTSQLSelectRow<TM>::clearParameters() {
	_realSQLExecute->clearParameters();
}

template<class TM>
void RTSQLSelectRow<TM>::bindout(const unsigned int pos, char& val, bool& isnull) {
	_realSQLExecute->bindout(pos, val, isnull);
}
template<class TM>
void RTSQLSelectRow<TM>::bindout(const unsigned int pos, signed char& val, bool& isnull) {
	_realSQLExecute->bindout(pos, val, isnull);
}
template<class TM>
void RTSQLSelectRow<TM>::bindout(const unsigned int pos, unsigned char& val, bool& isnull) {
	_realSQLExecute->bindout(pos, val, isnull);
}

template<class TM>
void RTSQLSelectRow<TM>::bindout(const unsigned int pos, short int& val, bool& isnull) {
	_realSQLExecute->bindout(pos, val, isnull);
}
template<class TM>
void RTSQLSelectRow<TM>::bindout(const unsigned int pos, unsigned short int& val, bool& isnull) {
	_realSQLExecute->bindout(pos, val, isnull);
}

template<class TM>
void RTSQLSelectRow<TM>::bindout(const unsigned int pos, int& val, bool& isnull) {
	_realSQLExecute->bindout(pos, val, isnull);
}
template<class TM>
void RTSQLSelectRow<TM>::bindout(const unsigned int pos, unsigned int& val, bool& isnull) {
	_realSQLExecute->bindout(pos, val, isnull);
}

template<class TM>
void RTSQLSelectRow<TM>::bindout(const unsigned int pos, long int& val, bool& isnull) {
	_realSQLExecute->bindout(pos, val, isnull);
}

template<class TM>
void RTSQLSelectRow<TM>::bindout(const unsigned int pos, unsigned long int& val, bool& isnull) {
	_realSQLExecute->bindout(pos, val, isnull);
}

template<class TM>
void RTSQLSelectRow<TM>::bindout(const unsigned int pos, float& val, bool& isnull) {
	_realSQLExecute->bindout(pos, val, isnull);
}
template<class TM>
void RTSQLSelectRow<TM>::bindout(const unsigned int pos, double& val, bool& isnull) {
	_realSQLExecute->bindout(pos, val, isnull);
}
template<class TM>
void RTSQLSelectRow<TM>::bindout(const unsigned int pos, long double& val, bool& isnull) {
	_realSQLExecute->bindout(pos, val, isnull);
}
template<class TM>
void RTSQLSelectRow<TM>::bindout(const unsigned int pos, std::string& val, bool& isnull) {
	_realSQLExecute->bindout(pos, val, isnull);
}



template<class TM>
void RTSQLSelectRow<TM>::execute(typename ResourceType<TM>::resource_t& hand) {
	_realSQLExecute->execute(hand);
}

template<class TM>
void RTSQLSelectRow<TM>::outputoperator(std::ostream& s) const {
	s << "(RT)";
	_realSQLExecute->outputoperator(s);
}

template<class TM>
const Clone* RTSQLSelectRow<TM>::getPlainSpecializedCommand() const {
	return _realSQLExecute->getPlainSpecializedCommand(); 
}

template class RTSQLSelectRow<DefaultThreadedModel>;
template void RTSQLSelectRow<DefaultThreadedModel>::bind<char>      (const unsigned int pos, const char& val, bool nul);
template void RTSQLSelectRow<DefaultThreadedModel>::bind<signed char> (const unsigned int pos, const signed char& val, bool nul);
template void RTSQLSelectRow<DefaultThreadedModel>::bind<unsigned char> (const unsigned int pos, const unsigned char& val, bool nul);

template void RTSQLSelectRow<DefaultThreadedModel>::bind<short>      (const unsigned int pos, const short& val, bool nul);
template void RTSQLSelectRow<DefaultThreadedModel>::bind<int>        (const unsigned int pos, const int& val, bool nul);
template void RTSQLSelectRow<DefaultThreadedModel>::bind<long>       (const unsigned int pos, const long& val, bool nul);

template void RTSQLSelectRow<DefaultThreadedModel>::bind<unsigned short>      (const unsigned int pos, const unsigned short& val, bool nul);
template void RTSQLSelectRow<DefaultThreadedModel>::bind<unsigned int>        (const unsigned int pos, const unsigned int& val, bool nul);
template void RTSQLSelectRow<DefaultThreadedModel>::bind<unsigned long>       (const unsigned int pos, const unsigned long& val, bool nul);

template void RTSQLSelectRow<DefaultThreadedModel>::bind<float>      (const unsigned int pos, const float& val, bool nul);
template void RTSQLSelectRow<DefaultThreadedModel>::bind<double>     (const unsigned int pos, const double& val, bool nul);
template void RTSQLSelectRow<DefaultThreadedModel>::bind<long double>     (const unsigned int pos, const long double& val, bool nul);

template void RTSQLSelectRow<DefaultThreadedModel>::bind<std::string>(const unsigned int pos, const std::string& val, bool nul);

} // namespace SQL
} // namespace fatalmind
