//********************************************************************
//*** ResourcePool/SQL/RTSQLSelectRow.t.cc
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: RTSQLSelectRow.t.cc,v 1.10 2009-03-14 15:12:25 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include "t/main.hh"
#include "config.h"
#include "ResourcePool/ResourcePool.hh"
#include "ResourcePool/SQL/ResourcePool.hh"

#include "testconfig.cc"

#include "SQLSelectRow.t.cc"

#ifdef HAVE_ORACLE
std::auto_ptr<RP> oci9_rp;
#endif
#ifdef HAVE_MYSQL
std::auto_ptr<RP> mysql3_rp;
#endif

void INIT_SQL_SQLSelect() {
#ifdef HAVE_ORACLE
	oci9_rp.reset(new RP(getFactory("oci9:" + getOracleLogon())));
	test_SQLSelectRow_init(*oci9_rp);
#endif
#ifdef HAVE_MYSQL
	mysql3_rp.reset(new RP(getFactory("mysql3:" + getMysqlLogon())));
	test_SQLSelectRow_init(*mysql3_rp);
#endif
}

void TEST_SQL_OCI9_SQLSelectRow_one() {
#ifdef HAVE_ORACLE
	test_SQLSelectRow_one(*oci9_rp);
#endif
}

void TEST_SQL_OCI9_SQLSelectRow_output() {
#ifdef HAVE_ORACLE
	test_SQLSelectRow_output(*oci9_rp);
#endif
}

void TEST_PERF_SQL_OCI9_SQLSelectRow_massbindout() {
#ifdef HAVE_ORACLE
	test_perf_SQLSelectRow_massbindout(*oci9_rp);
#endif
}

void TEST_PERF_SQL_OCI9_SQLSelectRow_one_spec_outer_C() {
#ifdef HAVE_ORACLE
	test_perf_SQLSelectRow_one_spec_outer(*oci9_rp);
#endif
}

void TEST_PERF_SQL_OCI9_SQLSelectRow_one_spec_inner_C() {
#ifdef HAVE_ORACLE
	test_perf_SQLSelectRow_one_spec_inner(*oci9_rp);
#endif
}

void TEST_PERF_SQL_OCI9_SQLSelectRow_first_one() {
#ifdef HAVE_ORACLE
	test_perf_SQLSelectRow_first_one(*oci9_rp);
#endif
}

void TEST_SQL_MYSQL3_SQLSelectRow_one() {
#ifdef HAVE_MYSQL
	test_SQLSelectRow_one(*mysql3_rp);
#endif
}

void TEST_SQL_MYSQL3_SQLSelectRow_output() {
#ifdef HAVE_MYSQL
	test_SQLSelectRow_output(*mysql3_rp);
#endif
}

void TEST_PERF_SQL_MYSQL_SQLSelectRow_massbindout() {
#ifdef HAVE_ORACLE
	test_perf_SQLSelectRow_massbindout(*mysql3_rp);
#endif
}

void TEST_PERF_SQL_MYSQL3_SQLSelectRow_one_spec_outer_C() {
#ifdef HAVE_MYSQL
	test_perf_SQLSelectRow_one_spec_outer(*mysql3_rp);
#endif
}

void TEST_PERF_SQL_MYSQL3_SQLSelectRow_one_spec_inner_C() {
#ifdef HAVE_MYSQL
	test_perf_SQLSelectRow_one_spec_inner(*mysql3_rp);
#endif
}

/* the sql sting is not mysql aware */
void x_TEST_PERF_SQL_MYSQL3_SQLSelectRow_first_one() {
#ifdef HAVE_MYSQL
	test_perf_SQLSelectRow_first_one(*mysql3_rp);
#endif
}


void CLEANUP_SQL_SQLSelectRow() {
#ifdef HAVE_ORACLE
	test_SQLSelectRow_cleanup(*oci9_rp);
#endif
#ifdef HAVE_MYSQL
	test_SQLSelectRow_cleanup(*mysql3_rp);
#endif
}

