//********************************************************************
//*** ResourcePool/SQL/RTSQLTransaction.hh
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: RTSQLTransaction.hh,v 1.5 2009-03-02 14:16:45 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_SQL_RTSQLTRANSACTION_HH
#define RESOURCEPOOL_SQL_RTSQLTRANSACTION_HH

#include "mm/gc_ptr.hh"
#include "ResourcePool/Command.hh"
#include "ResourcePool/SQL/Resource.hh"
#include "ResourcePool/ResourcePool.hh"

namespace fatalmind {
namespace SQL {

template<class TM> class RTSQLTransaction_Impl;

/**
 * This class implements the concept of a database transaction.
 * It takes a Command to be executed within a transaction.
 * In case you would like to put more commands into one transactions,
 * combine them first as BatchCommand and pass the BatchCommand to
 * the Transaction command.
 * The command will be executed within one transaction which
 * is automatically committed on success. On failure (every exception)
 * a rollback is performed.
 */
template<class TM>
class RTSQLTransaction
: public Command<ResourceType<TM> >
{
	public:
		typedef Command<ResourceType<TM> > innercommand_t;
		/**
		 * Creates a new Transaction object which manages the provided
		 * command.
		 */
		RTSQLTransaction(const ResourcePool<ResourceType<TM> >& p, const innercommand_t& cmd);
		RTSQLTransaction(const RTSQLTransaction<TM>& rho);
		virtual ~RTSQLTransaction();
		
		void execute(typename ResourceType<TM>::resource_t& hand);

	protected:
		virtual Clone* DoClone() const;

	private:
		typedef RTSQLTransaction_Impl<TM> rimpl_t;
		gc_ptr<rimpl_t> _realSQLTransaction;	

		typedef typename SQLFactory<TM>::databaseType_t databaseType_t;
};


} // namespace SQL
} // namespace fatalmind
#endif
