//********************************************************************
//*** ResourcePool/SQL/RTTypes.t.cc
//*** Copyright (c) 2003-2009 by Markus Winand <mws@fatalmind.com>
//*** $Id: RTTypes.t.cc,v 1.6 2009-03-02 14:16:45 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include <memory>
#include "t/main.hh"
#include "config.h"
#include "ResourcePool/ResourcePool.hh"
#include "ResourcePool/SQL/ResourcePool.hh"

#include "testconfig.cc"


#ifdef HAVE_ORACLE
std::auto_ptr<RP> oci9_p;
#endif
#ifdef HAVE_MYSQL
std::auto_ptr<RP> mysql3_p;
#endif

std::string getStringTable(RP& p) {
#ifdef HAVE_ORACLE
	if (&p == oci9_p.get()) {
		return "create table fatalmind_str (id numeric not null primary key, val  varchar2(512))";
	} 
#endif
	return "create table fatalmind_str (id numeric not null primary key, val  varchar(512))";
}

std::string getLobTable(RP& p) {
#ifdef HAVE_ORACLE
	if (&p == oci9_p.get()) {
		return "create table fatalmind_lob (id numeric not null primary key, val  clob)";
	} 
#endif
	return "create table fatalmind_lob (id numeric not null primary key, val  LONGBLOB)";
}
#include "Types.t.cc"

void INIT_SQL_TYPES() {
#ifdef HAVE_ORACLE
	oci9_p.reset(new RP(getFactory("oci9:" + getOracleLogon())));
	test_Types_init(*oci9_p);
#endif
#ifdef HAVE_MYSQL
	mysql3_p.reset(new RP(getFactory("mysql3:" + getMysqlLogon())));
	test_Types_init(*mysql3_p);
#endif
}

void TEST_SQL_OCI9_Types_int() {
#ifdef HAVE_ORACLE
	 test_Types_int(*oci9_p);
#endif
}
void TEST_SQL_MYSQL_Types_int() {
#ifdef HAVE_MYSQL
	 test_Types_int(*mysql3_p);
#endif
}

void TEST_SQL_OCI9_Types_float() {
#ifdef HAVE_ORACLE
	 test_Types_float(*oci9_p);
#endif
}
void TEST_SQL_MYSQL_Types_float() {
#ifdef HAVE_MYSQL
	 test_Types_float(*mysql3_p);
#endif
}

void TEST_SQL_OCI9_Types_double() {
#ifdef HAVE_ORACLE
	 test_Types_double(*oci9_p);
#endif
}
void TEST_SQL_MYSQL_Types_double() {
#ifdef HAVE_MYSQL
	 test_Types_double(*mysql3_p);
#endif
}

void TEST_SQL_OCI9_Types_string() {
#ifdef HAVE_ORACLE
	 test_Types_string(*oci9_p);
#endif
}

void TEST_SQL_OCI9_Types_crosstype_string() {
#ifdef HAVE_ORACLE
	 test_Types_crosstype_string(*oci9_p);
#endif
}
void TEST_SQL_MYSQL_Types_crosstype_string() {
#ifdef HAVE_MYSQL
	 test_Types_crosstype_string(*mysql3_p);
#endif
}

void TEST_SQL_OCI9_Types_string_crosstype() {
#ifdef HAVE_ORACLE
	 test_Types_string_crosstype(*oci9_p);
#endif
}
void TEST_SQL_MYSQL_Types_string_crosstype() {
#ifdef HAVE_MYSQL
	 test_Types_string_crosstype(*mysql3_p);
#endif
}

void TEST_SQL_MYSQL_Types_string() {
#ifdef HAVE_MYSQL
	 test_Types_string(*mysql3_p);
#endif
}

void TEST_SQL_OCI9_Types_lob() {
#ifdef HAVE_ORACLE
	 test_Types_lob(*oci9_p);
#endif
}
void TEST_SQL_MYSQL_Types_lob() {
#ifdef HAVE_MYSQL
	 test_Types_lob(*mysql3_p);
#endif
}

void TEST_PERF_SQL_OCI9_Types_lob_4k() {
#ifdef HAVE_ORACLE
	 test_perf_Types_lob_4k(*oci9_p);
#endif
}
void TEST_PERF_SQL_MYSQL_Types_lob_4k() {
#ifdef HAVE_MYSQL
	 test_perf_Types_lob_4k(*mysql3_p);
#endif
}

void TEST_PERF_SQL_OCI9_Types_lob_64k() {
#ifdef HAVE_ORACLE
	 test_perf_Types_lob_64k(*oci9_p);
#endif
}
void TEST_PERF_SQL_MYSQL_Types_lob_64k() {
#ifdef HAVE_MYSQL
	 test_perf_Types_lob_64k(*mysql3_p);
#endif
}

void TEST_PERF_SQL_OCI9_Types_lob_1024k() {
#ifdef HAVE_ORACLE
	 test_perf_Types_lob_1024k(*oci9_p);
#endif
}
void TEST_PERF_SQL_MYSQL_Types_lob_1024k() {
#ifdef HAVE_MYSQL
	 test_perf_Types_lob_1024k(*mysql3_p);
#endif
}

