#include "main.hh"
#include "config.h"

typedef unsigned int very_strange_fatalmind_typedef;

template<>
void _assertEqual<std::string>(
	  fatalmind::arg_traits<std::string>::const_param is
	, fatalmind::arg_traits<std::string>::const_param expect
	, const char* file
	, int line)
{
	ostringstream os;
	std::string::const_iterator eiis    = is.end();
	std::string::const_iterator eexpect = expect.end();
	std::string::const_iterator iis     = is.begin();
	std::string::const_iterator iexpect = expect.begin();
	for (; (iis != eiis) && (iexpect != eexpect); ++iis, ++iexpect) {
		if (*iis != *iexpect) {
			size_t pos = distance(is.begin(), iis);
			os << "assertEqual failed in " << file << ":"
				<< line << " (at position " << (pos) 
				<< " expected '" << *iexpect << "' (" 
					<< std::hex << std::setw(2) << (int(*iexpect)&0xFF)
				<< ") got '" << *iis << "' (" 
					<< std::hex << (int(*iis)&0xFF)
				<< ") actual result '" << is.substr(pos, min<size_t>(35, is.size()-pos)) << "'" << ((pos+35 < is.size())?"..." :"")<< ")";
			throw fatalmind::Exception(os.str());
		}
	}
	if (is.size() != expect.size()) {
		os << "assertEqual failed in " << file << ":"
			<< line << " (size is " << is.size() 
			<< " expected " << expect.size() << ")";
		throw fatalmind::Exception(os.str());
	}
}

void _assertEqual(const int is, const int expect, const char* file, int line) {
	_assertEqual<int>(is, expect, file, line);
}

void _assertEqual(const std::string& is, const std::string expect, const char* file, int line) {
	_assertEqual<std::string>(is, expect, file, line);
}
